/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.bescheid.attacheditem;

import static java.util.Objects.*;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;

import de.ozgcloud.bescheid.Bescheid;
import de.ozgcloud.bescheid.GrpcBescheid;
import de.ozgcloud.bescheid.GrpcSentInfo;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;

@Mapper(uses = { GrpcObjectMapper.class })
public interface AttachedItemMapper {

	AttachedItem mapFromVorgangAttachedItem(GrpcVorgangAttachedItem item);

	default GrpcBescheid toBescheid(AttachedItem bescheidItem) {
		var bescheidBuilder = GrpcBescheid.newBuilder().setVersion(bescheidItem.getVersion());
		Optional.ofNullable(bescheidItem.getId()).ifPresent(bescheidBuilder::setId);
		var bescheidData = bescheidItem.getItem();
		Optional.ofNullable(MapUtils.getString(bescheidData, Bescheid.FIELD_BESCHIEDEN_AM)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setBeschiedenAm);
		Optional.ofNullable(MapUtils.getBoolean(bescheidData, Bescheid.FIELD_BEWILLIGT)).ifPresent(bescheidBuilder::setBewilligt);
		Optional.ofNullable(MapUtils.getString(bescheidData, Bescheid.FIELD_BESCHEID_DOCUMENT)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setBescheidDocument);
		Optional.ofNullable(MapUtils.getString(bescheidData, Bescheid.FIELD_SEND_BY)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setSendBy);
		Optional.ofNullable(MapUtils.getString(bescheidData, Bescheid.FIELD_NACHRICHT_TEXT)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setNachrichtText);
		Optional.ofNullable(MapUtils.getString(bescheidData, Bescheid.FIELD_NACHRICHT_SUBJECT)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setNachrichtSubject);
		Optional.ofNullable(MapUtils.getString(bescheidData, Bescheid.FIELD_STATUS)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setStatus);
		Optional.ofNullable(bescheidData.get(Bescheid.FIELD_ATTACHMENTS)).map(this::toAttachments).ifPresent(bescheidBuilder::addAllAttachments);
		Optional.ofNullable(MapUtils.getMap(bescheidData, Bescheid.FIELD_SENT_INFO)).map(this::toSentInfo).ifPresent(bescheidBuilder::setSentInfo);
		return bescheidBuilder.build();
	}

	@SuppressWarnings("unchecked")
	default Collection<String> toAttachments(Object attachmentsObject) {
		if (attachmentsObject instanceof Collection<?> attachments) {
			return (Collection<String>) attachments;
		}
		if (attachmentsObject instanceof String attachment) {
			attachment = StringUtils.trimToNull(attachment);
			if (nonNull(attachment)) {
				return List.of(attachment);
			}
		} else if (nonNull(attachmentsObject)) {
			return List.of(String.valueOf(attachmentsObject));
		}
		return null; //NOSONAR
	}

	default GrpcSentInfo toSentInfo(Map<String, Object> sentInfoData) {
		return GrpcSentInfo.newBuilder()
				.setSentBy(MapUtils.getString(sentInfoData, Bescheid.FIELD_SENT_BY))
				.setSentAt(MapUtils.getString(sentInfoData, Bescheid.FIELD_SENT_AT)).build();
	}

}
