/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.bescheid.attributes;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.bescheid.BescheidCallContextAttachingInterceptor;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcAccessPermission;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;
import io.grpc.ClientInterceptor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service("bescheid_ClientAttributeRemoteService")
class ClientAttributeRemoteService {

	@GrpcClient("vorgang-manager")
	private ClientAttributeServiceBlockingStub serviceBlockingStub;

	@Autowired
	private ClientInterceptor bescheidCallContextInterceptor;

	public void setBooleanReadOnlyClientAttribute(String vorgangId, String attributeName, boolean value) {
		serviceBlockingStub.withInterceptors(bescheidCallContextInterceptor).set(buildRequest(vorgangId, attributeName, value));
	}

	GrpcSetClientAttributeRequest buildRequest(String vorgangId, String attributeName, boolean value) {
		return GrpcSetClientAttributeRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setAttribute(buildClientAttribute(attributeName, value))
				.build();
	}

	GrpcClientAttribute buildClientAttribute(String attributeName, boolean value) {
		return GrpcClientAttribute.newBuilder()
				.setClientName(BescheidCallContextAttachingInterceptor.BESCHEID_MANAGER_CLIENT_NAME)
				.setAccess(GrpcAccessPermission.READ_ONLY)
				.setAttributeName(attributeName)
				.setValue(GrpcClientAttributeValue.newBuilder().setBoolValue(value).build())
				.build();
	}
}
