/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.clientattribute;

import java.util.Optional;

import org.springframework.data.annotation.AccessType;
import org.springframework.data.annotation.PersistenceCreator;
import org.springframework.data.annotation.Transient;
import org.springframework.data.annotation.TypeAlias;

import jakarta.validation.constraints.NotNull;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.ToString;

@Builder(toBuilder = true)
@Getter
@AllArgsConstructor(access = AccessLevel.PRIVATE)
@ClientAttributeHasValueConstraint
@ToString
@TypeAlias("ClientAttribute")
public class ClientAttribute {

	public static final String FIELD_CLIENTNAME = "clientName";
	public static final String FIELD_ATTRIBUTENAME = "attributeName";
	public static final String FIELD_ACCESS = "access";
	public static final String FIELD_VALUE = "value";

	@PersistenceCreator
	ClientAttribute(AccessPermission access, Object value) {
		this.access = access;
		setValue(value);
	}

	public enum AccessPermission {
		PRIVATE, READ_ONLY, READ_WRITE;
	}

	@NotNull
	@Transient
	private String clientName;
	@Transient
	@NotNull
	private String attributeName;

	@Builder.Default
	private AccessPermission access = AccessPermission.PRIVATE;

	@Transient
	@Builder.Default
	@ToString.Exclude
	private Optional<String> stringValue = Optional.empty();
	@Transient
	@Builder.Default
	@ToString.Exclude
	private Optional<Long> intValue = Optional.empty();
	@Transient
	@Builder.Default
	@ToString.Exclude
	private Optional<Double> doubleValue = Optional.empty();
	@Transient
	@Builder.Default
	@ToString.Exclude
	private Optional<Boolean> boolValue = Optional.empty();

	@AccessType(AccessType.Type.PROPERTY)
	public Object getValue() {
		if (stringValue.isPresent()) {
			return stringValue.get();
		}
		if (intValue.isPresent()) {
			return intValue.get();
		}
		if (doubleValue.isPresent()) {
			return doubleValue.get();
		}
		if (boolValue.isPresent()) {
			return boolValue.get();
		}
		return null;
	}

	private void setValue(Object value) {
		stringValue = Optional.empty();
		intValue = Optional.empty();
		doubleValue = Optional.empty();
		boolValue = Optional.empty();

		if (value instanceof String stringVal) {
			stringValue = Optional.of(stringVal);
		} else if (value instanceof Long intVal) {
			intValue = Optional.of(intVal);
		} else if (value instanceof Double doubleVal) {
			doubleValue = Optional.of(doubleVal);
		} else if (value instanceof Boolean boolVal) {
			boolValue = Optional.of(boolVal);
		}
	}

}
